#ifndef __GL_WINDOW_H__
#define __GL_WINDOW_H__


#include "ngl/Camera.h"
#include "ngl/Colour.h"
#include "ngl/SpotLight.h"
#include "ngl/TransformStack.h"
#include "ngl/Obj.h"
// must be included after our stuff becuase GLEW needs to be first
#include <QtOpenGL>
#include "ngl/VertexArrayObject.h"

class GLWindow : public QGLWidget
{
Q_OBJECT        // must include this if you use Qt signals/slots
public :
  /// @brief Constructor for GLWindow
  /// @param [in] parent the parent window to create the GL context in
  GLWindow(
           QWidget *_parent
          );
  ~GLWindow();
private :

  /// @brief used to store the x rotation mouse value
  int m_spinXFace;
  /// @brief used to store the y rotation mouse value
  int m_spinYFace;
  /// @brief flag to indicate if the mouse button is pressed when dragging
  bool m_rotate;
  /// @brief the previous x mouse value
  int m_origX;
  /// @brief the previous y mouse value
  int m_origY;
  /// @brief Our Camera
  ngl::Camera *m_cam;
  /// @brief our transform stack for drawing the teapots
  ngl::TransformStack m_transformStack;
protected:

  /// @brief  The following methods must be implimented in the sub class
  /// this is called when the window is created
   void initializeGL();

  /// @brief this is called whenever the window is re-sized
  /// @param[in] _w the width of the resized window
  /// @param[in] _h the height of the resized window
  void resizeGL(
                const int _w,
                const int _h
               );
  /// @brief this is the main gl drawing routine which is called whenever the window needs to
  /// be re-drawn
  void paintGL();
	void buildVAO();

private :
		ngl::VertexArrayObject *m_vao;

  /// @brief this method is called every time a mouse is moved
  /// @param _event the Qt Event structure

  void mouseMoveEvent (
                       QMouseEvent * _event
                      );
  /// @brief this method is called everytime the mouse button is pressed
  /// inherited from QObject and overridden here.
  /// @param _event the Qt Event structure

  void mousePressEvent (
                        QMouseEvent *_event
                       );

  /// @brief this method is called everytime the mouse button is released
  /// inherited from QObject and overridden here.
  /// @param _event the Qt Event structure
  void mouseReleaseEvent (
                          QMouseEvent *_event
                         );

};

#endif
